# -*- coding: utf-8 -*-
#####################################################################
#
#       Author : Kushal Das
#       Copyright (c)  2009 Kushal Das
#       kushal@fedoraproject.org
#
#####################################################################

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sys
import xml.etree.ElementTree as ET
from optparse import OptionParser
import datetime

device = '/dev/ttyUSB0'

def get_datetime(date, time):
    """
    date: ddmmyy
    time: hhmm.ss.sss
    """
    return '20%s-%s-%sT%s:%s:%sZ' % (date[4:],date[2:4],date[:2],time[:2], \
                                    time[2:4], time[4:6])

def get_lat(data, direction):
    """
    data: String like '2500.8899'
    direction: 'N' or 'S'
    """
    if direction == 'N': return "%.6f" % (int(data[:2]) + float(data[2:])/60)
    return "-%.6f" % (int(data[:2]) + float(data[2:])/60)

def get_lon(data, direction):
    """
    data: String like '08807.8707'
    direction: 'E' or 'W'
    """
    if direction == 'E' : return "%.6f" % (int(data[:3]) + float(data[3:])/60)
    return "-%.6f" % (int(data[:3]) + float(data[3:])/60)


def create_track(data='', filename='test'):
    """
    This will parse the data and will create the gpx file for the same
    """
    root = ET.Element('gpx')
    root2 = ET.Element('gpx')
    time = ET.SubElement(root, 'time')
    time.text = "%s" % datetime.datetime.now()
    time2 = ET.SubElement(root2, 'time')
    time2.text = "%s" % datetime.datetime.now()

    trk = None
    trkseg = None

    for line in data:
        line = line.strip()
        parts = line.split(',')
        try:
            if line.startswith('T'):
                #Starting of new track
                trk = ET.SubElement(root,'trk')

                name = ET.SubElement(trk,'name')
                name.text = line[1:]

                trkseg = ET.SubElement(trk,'trkseg')

            elif len(parts) == 9:
                #Get the track details
                trkpt = ET.SubElement(trkseg, 'trkpt')
                trkpt.set('lat',get_lat(parts[2], parts[3]))
                trkpt.set('lon',get_lon(parts[4], parts[5]))

                ele = ET.SubElement(trkpt, 'ele')
                ele.text = parts[7]

                time = ET.SubElement(trkpt,'time')
                time.text = get_datetime(parts[6],parts[1])

            elif len(parts) == 10:
                #Get the waypoint details
                wpt = ET.SubElement(root2, 'wpt')
                wpt.set('lat',get_lat(parts[3], parts[4]))
                wpt.set('lon',get_lon(parts[5], parts[6]))

                ele = ET.SubElement(wpt, 'ele')
                ele.text = parts[8]

                name = ET.SubElement(wpt, 'name')
                name.text = parts[0][1:]

                desc = ET.SubElement(wpt,'desc')
                desc.text = get_datetime(parts[7],parts[2])

        except Exception, e:
            print "Error while parsing: %s" % line

    tree = ET.ElementTree(root)
    tree.write(filename + '.gpx')
    tree = ET.ElementTree(root2)
    tree.write(filename + '-way.gpx')

def read_device(device):
    """
    First run $stty -F device 9600
    Read the device and get raw log data
    """
    os.system('stty -F %s 9600' % device)
    dev = open(device,'w')
    data = []
    for line in dev.readline():
        data.append(line)
        if line.startswith('^GDLEND'):
            return ''.join(data)

def main(argv):
    """
    sparcy [OPTIONS] -n name of output file
    """
    parser = OptionParser(usage=main.__doc__)
    parser.add_option('-d', '--device', type="string",
                           help="path to device, default /dev/ttyUSB0 ")
    parser.add_option('-n', '--name', type='string',
                            help='name of the output file')
    parser.add_option('-f', '--file', type='string',
                            help='name of the input file')
    options, throaway = parser.parse_args(argv[1:])

    if options.file and options.name:
        if os.path.isfile(options.file):
            data = open(options.file).readlines()
            create_track(data, options.name)
        else:
            print "Wrong Input file"
    elif options.device and options.name:
        device = options.device
        data = read_device(device)
        #save the raw log
        raw_log = open(name + '-raw.txt','w')
        raw_log.write(data)
        raw_log.close()

        create_track(data, options.name)
    elif not options.name:
        print "Please give a proper output name with -n or --name"


if __name__ == '__main__':
    main(sys.argv)
